# Copyright (C) 1999, 2000 Jay Beale
# Licensed under the GNU General Public License

package Bastille::DNS;
use lib "/usr/lib";

use Bastille::API;
@ENV="";
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";

######## TO DO: Convert the chown line to API functions, so we don't
#########       have to use GLOBAL_PREFIX

#######################################################################
##                             DNS/BIND/NAMED                        ##
#######################################################################

&chrootBIND;
&DeactivateNamed;


sub chrootBIND {

    if ( $GLOBAL_CONFIG{"DNS"}{"chrootbind"} eq "Y" ) {
	&ActionLog("# sub chrootBIND\n");

	# Only chroot bind if we are running on a Red Hat / Mandrake system
	# and we haven't already chroot'ed it...

	my $home_dns="/home/dns";

        if (( not ( -d "${GLOBAL_PREFIX}$home_dns" ) ) and (($GLOBAL_DISTRO =~ /^RH/)  or ($GLOBAL_DISTRO =~ /^MN/) )) {
	
	    # We place BIND in a chroot'd environment, following the 
	    # instructions in SANS Step by Step 4.3.3

	    # Make the dns chroot directory
	    &B_create_dir ( $home_dns );      

	    # Create the dns user
	    &B_append_line ("/etc/passwd",":53:","dns:x:53:53::$home_dns:/bin/false\n");
	    &B_append_line ("/etc/group",":53:","dns:x:53:\n");
	    &B_append_line("/etc/shadow","^dns:","dns:*:11089:0:99999:7:::\n");

	    # Populate the dns chroot directory
	    &B_create_dir ("$home_dns/etc");
	    &B_chmod (0755,"$home_dns/etc");
	    &B_create_dir ("$home_dns/lib");
	    &B_chmod (0755,"$home_dns/lib");
	    &B_create_dir ("$home_dns/dev");
	    &B_chmod (0755,"$home_dns/dev");
	    &B_create_dir ("$home_dns/usr");
	    &B_chmod (0755,"$home_dns/usr");
	    &B_create_dir ("$home_dns/usr/sbin");
	    &B_chmod (0755,"$home_dns/usr/sbin");
	    &B_create_dir ("$home_dns/var");
	    &B_chmod (0755,"$home_dns/var");
	    &B_create_dir ("$home_dns/var/named");
	    &B_chmod (0755,"$home_dns/var/named");
	    &B_create_dir ("$home_dns/var/run");
	    &B_chmod (0755,"$home_dns/var/run");

	    unless ($GLOBAL_LOGONLY) {
		&B_mknod(" -m 666 ","$home_dns/dev/null"," c 1 3");

		&B_cp("/etc/named.conf","$home_dns/etc/named.conf");

		# Should we tell them to move their name logs over?

		&B_cp("/var/named/named.ca","$home_dns/var/named/named.ca");
		&B_cp("/var/named/named.local","$home_dns/var/named/named.local");

		`$GLOBAL_BIN{"chown"} -R dns.dns ${GLOBAL_PREFIX}$home_dns/var/named $home_dns/var/run`;
		&B_cp("/usr/sbin/named","$home_dns/usr/sbin/named");
		&B_chmod(0755,"$home_dns/usr/sbin/named");
		# Don't delete named -- let it get loaded from /usr/sbin, but
		# find data files in the chroot directory.  We'll still put 
		# a copy of the binary in the chroot directory so the named
		# binary can be reloaded from there if necessary.
		#
		#&B_delete_file("/usr/sbin/named");
                &B_cp("/usr/sbin/named-xfer","$home_dns/usr/sbin/named-xfer");
		&B_chmod(0755,"$home_dns/usr/sbin/named-xfer");
		&B_cp("/lib/libc.so.6","$home_dns/lib/libc.so.6");
		&B_cp("/lib/ld-linux.so.2","$home_dns/lib/ld-linux.so.2");
	    }

	    # named communicates normally with syslog via the device /dev/log, 
	    # which isn't accessible from the chroot'd environment.  We set 
	    # syslog to create and listen to a specific device just for bind.
	    
	    &B_replace_line("/etc/rc.d/init.d/syslog",'daemon syslogd -m 0\s*\$',"daemon syslogd -m 0 -a $home_dns/dev/log\n");
	
	    # Modify named's init script to use the chroot environment

	    unless ($GLOBAL_DISTRO eq "MN8.0") {
		    &B_replace_line("/etc/rc.d/init.d/named",'^\s*daemon\s+named',"daemon named -u dns -g dns -t $home_dns\n");
	    }
	    else {
		  # Mandrake 8.0 uses BIND 9, which deprecated the -g option.
		  &B_replace_line("/etc/rc.d/init.d/named",'^\s*daemon\s+named',"daemon named -u dns -t $home_dns\n");  
	    }

	}
        else {
	    # We only do this to Red Hat and Mandrake boxes for now, because
	    # we'd have to find all the libraries and modify startup files
	    # and this can just be totally difficult to automate when you
	    # don't know the target...
	    
	    &ErrorLog("Couldn't chroot named! Not on a Red Hat / Mandrake box or $home_dns already exists.\n");
	}
	
    } 
}


sub DeactivateNamed { 


    if ($GLOBAL_CONFIG{"DNS"}{"namedoff"} eq "Y") {

	&ActionLog("# sub DeactivateNamed\n");

	# Deactivate BIND unless it is being used.

	&B_chkconfig_off ("named");
    }

} 


1;





