# Copyright (C) 1999, 2000 Jay Beale
# Licensed under the GNU General Public License

package Bastille::FilePermissions;
use lib "/usr/lib";

use Bastille::API;
@ENV = "";
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";


## TO DO:
#
#    1) Re-write file permissions from scratch, possibly creating a 
#       smarter chmod for the API...
#    2) Re-do the SUID audit, based on Mandrake and other distros...
#


#######################################################################
##                            File Permissions                       ##
#######################################################################

&GeneralPerms;
#&Activatelibsafe;
&SUIDAudit;


sub GeneralPerms {

   &ActionLog("# sub GeneralPerms\n");

# First, we remove world read,execute,write access from some utitilies
# that ordinary users shouldn't need.  

#
# Remove world read, write, execute where it's unnecessary
# This borrowed from a SANS publication, which borrowed/modified it
# from a TrinityOS script.
# 

   my $distro=$GLOBAL_DISTRO;

   if ($GLOBAL_CONFIG{"FilePermissions"}{"generalperms_1.1"} eq "Y") {
       if ($distro =~ /^RH/ or $distro =~ /^MN/) {
	   &B_chmod_if_exists(0700,"/bin/linuxconf");
	   &B_chmod_if_exists(0750,"/bin/mt");
	   &B_chmod_if_exists(0750,"/bin/setserial");
	   &B_chmod_if_exists(0750,"/sbin/badblocks");
	   &B_chmod_if_exists(0750,"/sbin/ctrlaltdel");
	   &B_chmod_if_exists(0750,"/sbin/chkconfig");
	   &B_chmod_if_exists(0750,"/sbin/debugfs");
	   &B_chmod_if_exists(0750,"/sbin/depmod");
	   &B_chmod_if_exists(06750,"/sbin/dump");
	   &B_chmod_if_exists(0750,"/sbin/dumpe2fs");
	   ####### Are there other dump___ programs to worry about?
	   &B_chmod_if_exists(0750,"/sbin/fdisk");
	   &B_chmod_if_exists(0750,"/sbin/fsck");
	   &B_chmod_if_exists(0750,"/sbin/fsck.ext2");
	   &B_chmod_if_exists(0750,"/sbin/fsck.minix");
	   &B_chmod_if_exists(0750,"/sbin/ftl_check");
	   &B_chmod_if_exists(0750,"/sbin/ftl_format");
	   &B_chmod_if_exists(0750,"/sbin/halt");
	   &B_chmod_if_exists(0750,"/sbin/hdparm");
	   &B_chmod_if_exists(0750,"/sbin/hwclock");
	   &B_chmod_if_exists(0750,"/sbin/ifconfig");
	   &B_chmod_if_exists(0750,"/sbin/ifdown");
	   &B_chmod_if_exists(0750,"/sbin/ifport");
	   &B_chmod_if_exists(0750,"/sbin/ifup");
	   &B_chmod_if_exists(0750,"/sbin/ifuser");
	   &B_chmod_if_exists(0750,"/sbin/init");
	   &B_chmod_if_exists(0750,"/sbin/insmod");
	   &B_chmod_if_exists(0750,"/sbin/isapnp");
	   &B_chmod_if_exists(0750,"/sbin/kerneld");
	   &B_chmod_if_exists(0750,"/sbin/killall5");
	   &B_chmod_if_exists(0750,"/sbin/lilo");
	   &B_chmod_if_exists(0750,"/sbin/mingetty");
	   &B_chmod_if_exists(0750,"/sbin/mkbootdisk");
	   &B_chmod_if_exists(0750,"/sbin/mke2fs");
	   &B_chmod_if_exists(0750,"/sbin/mkfs");
	   &B_chmod_if_exists(0750,"/sbin/mkfs.ext2");
	   &B_chmod_if_exists(0750,"/sbin/mkfs.minix");
	   &B_chmod_if_exists(0750,"/sbin/mkfs.msdos");
	   &B_chmod_if_exists(0750,"/sbin/mkinitrd");
	   &B_chmod_if_exists(0750,"/sbin/mkpv");
	   &B_chmod_if_exists(0750,"/sbin/mkraid");
	   &B_chmod_if_exists(0750,"/sbin/mkswap");
	   &B_chmod_if_exists(0750,"/sbin/modinfo");
	   &B_chmod_if_exists(0750,"/sbin/modprobe");
	   &B_chmod_if_exists(02750,"/sbin/netreport");

	   &B_chmod_if_exists(0750,"/sbin/pnpdump");
	   &B_chmod_if_exists(0750,"/sbin/portmap");
	   &B_chmod_if_exists(0750,"/sbin/quotaon");
	   &B_chmod_if_exists(06750,"/sbin/restore");
	   &B_chmod_if_exists(0750,"/sbin/runlevel");
	   &B_chmod_if_exists(0750,"/sbin/stinit");
	   &B_chmod_if_exists(0750,"/sbin/swapon");
	   &B_chmod_if_exists(0750,"/sbin/tune2fs");
	   &B_chmod_if_exists(0750,"/sbin/uugetty");
	   
	   # Comanche was removed from RH6.1 -- don't try to chmod if it doesn't exist
	   if ( $distro ne "RH6.0" and $distro ne "MN6.0") {
	       &B_chmod_if_exists(0750,"/usr/bin/comanche");
	   }
	   &B_chmod_if_exists(0750,"/usr/bin/control-panel");
	   &B_chmod_if_exists(0750,"/usr/bin/eject");
	   &B_chmod_if_exists(04750,"/usr/bin/gpasswd");
	   &B_chmod_if_exists(0750,"/usr/bin/kernelcfg");
	   
	   # Deviate from SAN/Trinity script.  Unless we create special 
	   # gnome/xwindows group, removing user access to gnome utilities 
	   # breaks things...
	   
	   # Don't modify any gnome stuff...

	   # Deviate from the SANS/Trinity script here: ordinary users should
	   # not be able to start the news server.  ( Chris Owen )
	   &B_chmod_if_exists(0500,"/usr/bin/inndstart");
	   &B_chmod_if_exists(0500,"/usr/bin/startinnfeed");
	   
	   &B_chmod_if_exists(0755,"/usr/bin/lpq","/usr/bin/lpqall.faces","/usr/bin/lprm");
	   &B_chmod_if_exists(0755,"/usr/bin/lptest","/usr/bin/lpunlock");
	   &B_chmod_if_exists(04755,"/usr/bin/lpr");
	   
	   &B_chmod_if_exists(0750,"/usr/bin/minicom");
	   &B_chmod_if_exists(0750,"/usr/bin/netcfg");
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/atd");
	   &B_chmod_if_exists(0750,"/usr/sbin/atrun");
	   &B_chmod_if_exists(0750,"/usr/sbin/crond");
	   &B_chmod_if_exists(0750,"/usr/sbin/dhcpd");
	   &B_chmod_if_exists(0750,"/usr/sbin/dhcrelay");
	   &B_chmod_if_exists(0750,"/usr/sbin/edquota");
	   &B_chmod_if_exists(0750,"/usr/sbin/exportfs");
	   &B_chmod_if_exists(0750,"/usr/sbin/groupadd");
	   &B_chmod_if_exists(0750,"/usr/sbin/groupdel");
	   &B_chmod_if_exists(0750,"/usr/sbin/groupmod");
	   &B_chmod_if_exists(0750,"/usr/sbin/grpck");
	   &B_chmod_if_exists(0750,"/usr/sbin/grpconv");
	   &B_chmod_if_exists(0750,"/usr/sbin/grpunconv");
	   &B_chmod_if_exists(0750,"/usr/sbin/imapd");
	   &B_chmod_if_exists(0750,"/usr/sbin/in.comsat");
	   &B_chmod_if_exists(0750,"/usr/sbin/in.fingerd");
	   &B_chmod_if_exists(0750,"/usr/sbin/in.identd");
	   &B_chmod_if_exists(0750,"/usr/sbin/in.ntalkd");
	   
	   # deviate from the SANS script for the in.r__d daemons -- these are 
	   # bad news!
	   &B_chmod_if_exists(0000,"/usr/sbin/in.rexecd");
	   &B_chmod_if_exists(0000,"/usr/sbin/in.rlogind");
	   &B_chmod_if_exists(0000,"/usr/sbin/in.rshd");
	   &B_chmod_if_exists(0750,"/usr/sbin/in.telnetd");
	   
	   # deviate from the SANS script -- tftpd has had serious vulnerabilities.
	   &B_chmod_if_exists(0000,"/usr/sbin/in.tftpd");
	   &B_chmod_if_exists(0750,"/usr/sbin/in.timed");
           &B_chmod_if_exists(0750,"/usr/sbin/inetd");
	   &B_chmod_if_exists(0750,"/usr/sbin/ipop2d");
	   &B_chmod_if_exists(0750,"/usr/sbin/ipop3d");
	   
	   # klogd was moved to /sbin in RH6.1, so modify the right file
	   &B_chmod_if_exists(0750,"/usr/sbin/klogd");
	   &B_chmod_if_exists(0750,"/sbin/klogd");
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/logrotate");
	   &B_chmod_if_exists(02750,"/usr/sbin/lpc");
	   &B_chmod_if_exists(0740,"/usr/sbin/lpd");
	   &B_chmod_if_exists(0750,"/usr/sbin/lpf");
	   &B_chmod_if_exists(0755,"/usr/sbin/lsof");
	   &B_chmod_if_exists(0550,"/usr/sbin/makemap");
	   &B_chmod_if_exists(0750,"/usr/sbin/mouseconfig");
	   &B_chmod_if_exists(0750,"/usr/sbin/named");
	   &B_chmod_if_exists(0750,"/usr/sbin/named-xfer");
	   &B_chmod_if_exists(0750,"/usr/sbin/newusers");
	   &B_chmod_if_exists(0750,"/usr/sbin/nmbd");
	   &B_chmod_if_exists(0750,"/usr/sbin/ntpdate");
	   &B_chmod_if_exists(0750,"/usr/sbin/ntpq");
	   &B_chmod_if_exists(0750,"/usr/sbin/ntptime");
	   &B_chmod_if_exists(0750,"/usr/sbin/ntptrace");
	   &B_chmod_if_exists(0750,"/usr/sbin/ntsysv");
	   &B_chmod_if_exists(0750,"/usr/sbin/pppd");
	   &B_chmod_if_exists(0750,"/usr/sbin/pwck");
	   &B_chmod_if_exists(0750,"/usr/sbin/pwconv");
	   &B_chmod_if_exists(0750,"/usr/sbin/pwunconv");
	   &B_chmod_if_exists(0550,"/usr/sbin/quotastats");
	   &B_chmod_if_exists(0750,"/usr/sbin/rdev");
	   &B_chmod_if_exists(0550,"/usr/sbin/repquota");
	   &B_chmod_if_exists(0750,"/usr/sbin/rotatelogs");
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.bootparamd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.mountd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.nfsd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.rquotad");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.rstatd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.rusersd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.rwalld");
	   
	   # rpc.statd was moved to /sbin in RH6.1, so modify the right file 
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.statd");
	   &B_chmod_if_exists(0750,"/sbin/rpc.statd");
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.yppasswdd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpc.ypxfrd");
	   &B_chmod_if_exists(0750,"/usr/sbin/rpcinfo");
	   &B_chmod_if_exists(0750,"/usr/sbin/samba");
	   &B_chmod_if_exists(0750,"/usr/sbin/setup");
	   &B_chmod_if_exists(0750,"/usr/sbin/showmount");
	   &B_chmod_if_exists(0750,"/usr/sbin/smbd");
	   &B_chmod_if_exists(0750,"/usr/sbin/squid");
	   
	   # syslogd was moved to /sbin in RH6.1, so modify the right file
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/syslogd");
	   &B_chmod_if_exists(0750,"/sbin/syslogd");
	   
	   &B_chmod_if_exists(0750,"/usr/sbin/taper");
	   &B_chmod_if_exists(0750,"/usr/sbin/tcpd");
	   &B_chmod_if_exists(0750,"/usr/sbin/tcpdchk");
	   &B_chmod_if_exists(0750,"/usr/sbin/tcpdmatch");
	   &B_chmod_if_exists(0750,"/usr/sbin/tcpdump");
	   &B_chmod_if_exists(0750,"/usr/sbin/timeconfig");
	   &B_chmod_if_exists(0750,"/usr/sbin/timed");
	   &B_chmod_if_exists(0750,"/usr/sbin/tmpwatch");
	   &B_chmod_if_exists(0750,"/usr/sbin/tunelp");
	   &B_chmod_if_exists(0750,"/usr/sbin/useradd");
	   &B_chmod_if_exists(0750,"/usr/sbin/userdel");
	   &B_chmod_if_exists(04750,"/usr/sbin/userhelper");
	   &B_chmod_if_exists(0750,"/usr/sbin/usermod");
	   &B_chmod_if_exists(04750,"/usr/sbin/usernetctl");
	   &B_chmod_if_exists(0750,"/usr/sbin/vipw");
	   &B_chmod_if_exists(0750,"/usr/sbin/xntpd");
	   &B_chmod_if_exists(0750,"/usr/sbin/xntpdc");
       }
       else {
	   &ErrorLog("Didn't carry out general permissions modifications as they are very specific to Red Hat / Mandrake\n");
       }
   }
   if ($GLOBAL_CONFIG{"generalperms_1.2_mandrake"}) {
       &ActionLog("# sub GeneralPerms -- 1.2_mandrake");

       my $security_level = $GLOBAL_CONFIG{"generalperms_1.2_mandrake"};

       # Check the security level
       my $security_level_is_ok = 0;
       my $lvl;
       foreach $lvl (0,1,2,3,4,5) {
	   if ($lvl = $security_level) {
	       $security_level_is_ok =1;
	   }
       }

       # Change permissions according to Mandrake's level files
       # BUG!  This doesn't respect our B_PREFIX at all!
       unless ($security_level_is_ok) { $security_level = 3; }
       my $return = open PERMFILE, "/usr/share/Bastille/mandrake_perm.$security_level";
       if ($return) {
	   while (my $line = <PERMFILE>) {
	       if ($line = /^(.*)\s+([^\.]+).([^\.]+)\s+(\d+)/) {
		   my $file=$1;
		   my $owner=$2;
		   my $group=$3;
		   my $mode=$4;

		   # BUG: may get wrong value if $GLOBAL_PREFIX is not ''
		   my $uid = (getpwnam($owner))[2];
		   my $gid = (getgrnam($group))[2];
		   
		   &B_chown($uid,$file);
		   &B_chgrp($gid,$file);
		   &B_chmod_if_exists($mode,$file);
	       }
	   }
       }
       else {
	   &ErrorLog("Failed to open permfile /usr/share/Bastille/mandrake_perm.$security_level\nFile permissions not changed\n");
       }

   }

}


sub Activatelibsafe {

    &ActionLog("# sub Activatelibsafe\n");
    
    if ($GLOBAL_CONFIG{"FilePermissions"}{"libsafe"} eq "Y") {

	&B_append_line("/etc/ld.so.preload","/lib/libsafe.so.1.3\n");
	
    }
}


sub SUIDAudit {
   &ActionLog("# sub SUIDAudit\n");
##
### SUID Audit / Correction
##


# First, we list all SUID ROOT programs in the Redhat 6.0 install.
# We include the full list here, at least during the development 
# cycle of this script, to 1) allow easy edits and 2) maintain an 
# easy-to-find list.  Comment lines with names of binaries, thus, are
# just listing a non-modified binary.

#/bin/login
#/bin/su
#	   

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidmount"} eq "Y") {
       &B_chmod_if_exists(0700,"/bin/mount");
       &B_chmod_if_exists(0700,"/bin/umount");
   }

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidping"} eq "Y") {
       &B_chmod_if_exists(0700,"/bin/ping");
   }
    
#/sbin/pwdb_chkpwd

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suiddump"} eq "Y") {
       &B_chmod_if_exists(0700,"/sbin/dump");
       &B_chmod_if_exists(0700,"/sbin/restore");
   }

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidcard"} eq "Y") {
      &B_chmod_if_exists(0700,"/sbin/cardctl");
   }
    
#/usr/X11R6/bin/Xwrapper
    
   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidat"} eq "Y") {
       &B_chmod_if_exists(0700,"/usr/bin/at");
   }

#/usr/bin/crontab
#/usr/bin/chage
#/usr/bin/gpasswd
#/usr/bin/chfn
#/usr/bin/chsh
#/usr/bin/newgrp 

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suiddos"} eq "Y") {
       &B_chmod_if_exists(0700,"/usr/bin/dos");
   }

#/usr/bin/disable-paste ---?????????

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidnews"} eq "Y") {
       &B_chmod_if_exists(0700, "/usr/bin/inndstart","/usr/bin/startinnfeed");
   }

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidprint"} eq "Y") {
       &B_chmod_if_exists(0700,"/usr/bin/lpq","/usr/bin/lpr","/usr/bin/lprm");
   }

#/usr/bin/passwd
#/usr/bin/suidperl
#/usr/bin/sperl5.00503
#/usr/bin/procmail

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidrtool"} eq "Y") {
       &B_chmod_if_exists(000,"/usr/bin/rcp","/usr/bin/rlogin","/usr/bin/rsh");
   }

#/usr/bin/zgv --- ????????? why does this have suid root?
#/usr/bin/ssh1

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidusernetctl"} eq "Y") {
       &B_chmod_if_exists(0700,"/usr/sbin/usernetctl");
   }
    
#/usr/sbin/sendmail

   if ($GLOBAL_CONFIG{"FilePermissions"}{"suidtrace"} eq "Y") {
       &B_chmod_if_exists(0700, "/usr/sbin/traceroute");
   }

#/usr/sbin/userhelper
#/usr/libexec/pt_chown

}    

1;

